<?php

namespace App;

use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Model;

/**
 * Base model for all TGenBase classes
 * Implements versioned queries and insert-only logic
 */
class BaseModelVersioned extends Model
{
  /**
   * The primary key for the model.
   *
   * @var string
   */
  protected $primaryKey = 'Id';

  const CREATED_AT = 'CreatedAt';
  const UPDATED_AT = 'UpdatedAt';

  /**
   * The storage format of the model's date columns.
   *
   * @var string
   */
  protected $dateFormat = 'U';

  /**
   * Model attributes with defailt values
   *
   * @var array
   */
  protected $attributes = [
    'Id' => -1,
    'CreatedAt' => 0,
  ];

  /**
   * The attributes that are mass assignable.
   *
   * @var array
   */
  protected $fillable = ['Id', 'CreatedAt'];

  /**
   * Indicates if created_at and updated_at timestamps are used
   *
   * @var bool
   */
  public $timestamps = false;

  /**
   * Indicates if the IDs are auto-incrementing.
   *
   * @var bool
   */
  public $incrementing = false;

  /**
   * Table name for this model
   *
   * @var strung
   */
  protected $table = '%BaseModelVersioned';

  /**
   * Default column for dictionary building
   *
   * @var string
   */
  public const dictionaryColumn = '%&';

  /**
   * Indicates if audit enabled for this class.
   *
   * @var bool
   */
  public const audit = false;

  /**
   * The attributes that should be hidden for arrays.
   *
   * @var array
   */
  protected $hidden = ['SeqNo', 'pivot'];

  /**
   * Database Query Builder scope for versioned entities
   *
   * @param  \Illuminate\Database\Query\Builder  $query
   * @param  int  $rid
   * @return \Illuminate\Database\Query\Builder
   */
  public function scopeVersioned($query, $rid) {
    return $query->where('CreatedAt', '<=', $rid)->whereIn('SeqNo', function($subQuery) use ($query) {
      $subQuery->selectRaw('MAX("SeqNo")')->from($this->table)->groupBy('Id');
    });
  }

  /**
   * Retreive next Id for the versioned entities
   *
   * @return int
   */
  public static function GetNextId() {
    $ret = static::select('Id')->orderBy('Id', 'DESC')->limit(1)->value('Id');
    return $ret === null ? 1 : $ret + 1;
  }

  /**
   * Database Query Builder for versioned entities with version Id
   *
   * @param  int|null  $rid
   * @return \Illuminate\Database\Query\Builder
   */
  public static function GetBy($rid=null) {
    if ($rid === null) $rid = now()->format("U");
    $with = request('With', null);
    if ($with) {
      if (is_string($with))
        $with = [$with];

      $filtered = [];
      foreach ($with as $key => $value) {
        if (method_exists(static::class, explode(":", $value)[0])) {
          $filtered[] = $value;
        }
      }

      return static::versioned($rid)->with($filtered);
    }
    else
      return static::versioned($rid);
  }

  /**
   * Get versioned entity by Id with version Id
   *
   * @param  int  $Id
   * @param  int|null  $rid
   * @return \App\BaseModel|mixed
   */
  public static function GetById($Id, $rid=null) {
    return Cache::remember(static::class, 'GetById', $Id.$rid, 600, function () use ($Id, $rid) {
      return static::GetBy($rid)->where('Id', $Id)->get()->first();
    });
  }

  /**
   * Get all versioned entities with version Id
   *
   * @param  int|null  $rid
   * @return array
   */
  public static function GetAll($rid=null) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetAll', [$perPage,$page,$rid], 600, function () use ($perPage, $rid) {
      if (is_null($perPage))
        return static::GetBy($rid)->get();
      else
        return static::GetBy($rid)->paginate($perPage);
    });
  }

  /**
   * Get all versions of a versioned entity with Id
   *
   * @param  int  $Id
   * @return array
   */
  public static function GetAllVersions($Id) {
    return Cache::remember(static::class, 'GetAllVersions', $Id, 600, function () use ($Id) {
      return static::where('Id', $Id)->orderBy('CreatedAt')->get();
    });
  }

  /**
   * Get array of versioned entities with array of Ids and version Id
   *
   * @param  array  $Ids
   * @return array
   */
  public static function GetArray($Ids, $rid=null) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetArray', [$Ids,$perPage,$page,$rid], 600, function () use ($Ids, $perPage, $rid) {
      if (is_null($perPage))
        return static::GetBy($rid)->whereIn('Id', $Ids)->get();
      else
        return static::GetBy($rid)->whereIn('Id', $Ids)->paginate($perPage);
    });
  }

  /**
   * Get array of entities with array of Ids
   *
   * @param  int|null $Query
   * @param  int|null  $rid
   * @return array
   */
  public static function GetWithQuery($Query, $rid=null) {
    $Query = collect($Query);
    \App\QueryBuilder::check($Query);

    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetWithQuery', [$Query,$perPage,$page,$rid], 600, function () use ($Query, $perPage, $rid) {
      $ret = static::GetBy($rid);
      \App\QueryBuilder::process($Query, $ret);
      if (is_null($perPage))
        return $ret->get();
      else
        return $ret->paginate($perPage);
    });
  }

  /**
   * Get dictionary of 'Id' => $Column as 'Id' => 'Name' for this entity
   *
   * @param  string $Column
   * @param  int|null  $rid
   * @return array
   */
  public static function GetDictionary($Column, $rid=null) {
    if (is_null($Column) || $Column === '')
      $Column = static::dictionaryColumn;

    return Cache::remember(static::class, 'GetDictionary', $Column.$rid, 600, function () use ($Column, $rid) {
      $inst = new static;
      if (in_array($Column, $inst->getHidden()))
        $Column = '%&'.$Column;

      $dictionary = [];
      static::GetBy($rid)->get(['Id', $Column])->each(function($val) use ($Column, &$dictionary) {
        $dictionary[$val['Id']] = $val[$Column];
      });
      return $dictionary;
    });
  }

  /**
   * Store an instance of versioned entity with version Id
   * If entity Id is -1 a new Id will be allocated
   *
   * @param  int|null  $rid
   * @return \App\BaseModel|mixed
   */
  public function Store($rid=null) {
    if ($rid === null) $rid = now()->format("U");
    $this->CreatedAt = $rid;
    $this->exists = false;
    unset($this->SeqNo);

    Cache::flush(static::class);

    if (!app()->runningInConsole() && static::audit) {
      $this->CreatedByUserId = auth()->user()->Id;
    }

    if (!isset($this->Id) || is_null($this->Id) || $this->Id == -1) {
      /// sqlite does not support insertGetId with custom column
      if (env('DB_CONNECTION', 'sqlite') === 'sqlite') {
        return DB::transaction(function () {
          $this->Id = static::GetNextId();
          $this->save();

          return $this;
        }, 3);
      } else {
        $next = $this->selectRaw('COALESCE(MAX("Id") + 1,1)')->from($this->table)->toSql();
        $attrs = $this->getAttributes();
        $attrs["Id"] = DB::raw("($next)");
        $this->Id = $this->insertGetId($attrs, "Id");
        return $this;
      }
    } else {
      $this->save();
      return $this;
    }
  }

  /**
   * Store an eloquent collection of versioned entity objects with version Id
   * If any entity Id is -1 a new Id will be allocated
   *
   * @param  \Illuminate\Support\Collection  $Collection
   * @param  int|null  $rid
   * @return null  nothing
   */
  public static function StoreCollection($Collection, $rid=null) {
    if (!$Collection->count()) {
      return collect([]);
    }

    if ($rid === null) $rid = now()->format("U");
    Cache::flush(static::class);

    DB::transaction(function () use (&$Collection, $rid) {
      $nextId = 1;
      if ($Collection->contains('Id', '-1')) {
        $nextId = static::GetNextId();
      }

      $Collection = $Collection->map(function ($val) use (&$nextId, $rid) {
        $val->CreatedAt = $rid;
        $val->exists = false;
        unset($val->SeqNo);

        if (!isset($val->Id) || is_null($val->Id) || $val->Id == -1) {
          $val->Id = $nextId++;
        }

        if (!app()->runningInConsole() && static::audit) {
          $val->CreatedByUserId = auth()->user()->Id;
        }

        return $val->getAttributes();
      });

      static::insert($Collection->toArray());

      return $Collection;
    }, 3);

    return null;
  }

  /**
   * Store an array of versioned entity associated arrays with version Id
   * If any entity Id is -1 a new Id will be allocated
   *
   * @param  array  $Array
   * @param  int|null  $rid
   * @return array
   */
  public static function StoreArray($Array, $rid=null) {
    return static::StoreCollection(collect($Array), $rid);
  }

  public function delete() {
    throw new Exception('Deleting is not allowed.');
  }

  public function update(array $attributes = [], array $options = []) {
    throw new Exception('Updating is not allowed.');
  }
}
