<?php

namespace App\Http\Controllers;

use Illuminate\Foundation\Bootstrap\ErrorException;
use Adldap\Connections\ConnectionException;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Adldap\Laravel\Facades\Adldap;

use App\Http\Controllers\BaseController;
use App\Http\Controllers\BaseControllerVersioned;

class UserController extends BaseControllerVersioned
{
  protected $Model = 'App\User';

  public function Login(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'email' => 'required',
      'password' => 'required',
    ]);

    $user = null;

    try {
      if (env('USE_LDAP', false)) {
        $adldap = Adldap::getFacadeRoot();
        $provider = $adldap->getDefaultProvider();

        $email = request('email');
        $password = request('password');
        $cn = 'cn='.$email.','.$provider->getConfiguration()->get('base_dn');
        $provider->connect($cn, $password);
        if ($provider->getConnection()->isBound()) {
          $needsUpdate = false;
          $user = \App\User::GetByEmail($email)->first();

          // import user from ldap, if it does not exist locally
          if (is_null($user)) {
            $needsUpdate = true;
            $user = new \App\User();
            $user->Id = -1;
            $user->Email = $email;

            $role = env('LDAP_DEFAULT_USER_ROLE', 10);
            if ($role > 100) $role = 100;
            $user->Role = $role;
            $user->Status = 2;
          }

          // check for changes against ldap
          $ldap_user = $provider->search()->users()->whereEquals('cn', $email)->first();

          $fullName = $ldap_user->getFirstAttribute('givenname').' '.
                            $ldap_user->getFirstAttribute('sn');
          if ($fullName !== $user->FullName) {
            $needsUpdate = true;
            $user->FullName = $fullName;
          }

          $address = $ldap_user->getFirstAttribute('o').', '.
                     $ldap_user->getFirstAttribute('l').', '.
                     $ldap_user->getFirstAttribute('c');

          if ($address !== $user->Address) {
            $user->Address = $address;
          }

          if (!Hash::check(request('password'), $user->Password)) {
            $needsUpdate = true;
            $user->Password = Hash::make(request('password'));
          }

          // update if needed
          if ($needsUpdate) {
            $user->Store();
          }
        }
      }
    } catch (\Exception $e) { error_log($e); }

    if (is_null($user)) {
      $provider = new \App\Providers\TokenUserProvider();
      $user = $provider->retrieveByCredentials([
        'Email' => request('email'),
        'Password' => request('password'),
      ]);
    }

    if (is_null($user)) {
      return \App\Http\Response::Error("Authorization error", 401);
    }

    $token_str = Str::random(50);
    $token = new \App\AuthToken($token_str);
    $user->AuthTokens()->save($token);

    return \App\Http\Response::Data([
      'auth-token' => $token_str,
      'auth-user' => $user,
    ]);
  }

  /**
   * Local user registration
   * LDAP user import happens on login, see UserController::Login
   *
   * param  Request $request
   * @return array
   */
  public function RegisterUser(Request $request) {
    if (!env('USE_REGISTRATION', false)) {
      throw new \App\Exceptions\AppException(422, 'User registrations were disabled by administrator');
    }

    $data = ["user" => [
              "Role" => env("REGISTRATION_DEFAULT_USER_ROLE", 10),
              "Status" => env("REGISTRATION_DEFAULT_USER_STATUS", 2),
            ]];
    $data = array_merge_recursive($request->json()->all(), $data);
    $request->json()->replace($data);

    return (new \App\Http\Controllers\AdminController)->RegisterUser($request);
  }

  public function Logout(Request $request) {
    \App\AuthToken::GetWithToken($request->headers->get('auth-token'))->DeleteAndForget();

    return \App\Http\Response::Data(null);
  }

  public function Get(Request $request) {
    return \App\Http\Response::Data(auth()->user());
  }

  public function ChangeUserPassword(Request $request) {
    return (new \App\Http\Controllers\AdminController)->ChangeUserPassword($request);
  }

  public function ChangeUser(Request $request) {
    return (new \App\Http\Controllers\AdminController)->ChangeUser($request);
  }

  public function GetByFullName(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'FullName' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByFullName(request('FullName'), request('Rid')));
  }

  public function GetByAddress(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Address' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByAddress(request('Address'), request('Rid')));
  }

  public function GetByEmail(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Email' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByEmail(request('Email'), request('Rid')));
  }

  public function GetByRole(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Role' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByRole(request('Role'), request('Rid')));
  }

  public function GetByStatus(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Status' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByStatus(request('Status'), request('Rid')));
  }

  public function GetByCreatedAt(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'CreatedAt' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByCreatedAt(request('CreatedAt'), request('Rid')));
  }

  public function GetByUpdatedAt(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'UpdatedAt' => 'required',
      'Rid' => 'nullable|numeric',
    ]);

    return \App\Http\Response::Data($this->Model::GetByUpdatedAt(request('UpdatedAt'), request('Rid')));
  }
}
