import Property from '@/js/Property.js'
import Relation from '@/js/Relation.js'
import Static from '@/js/Static.js'

class ClassDef {
  constructor(object) {
    if (!object)
      object = {};
    this.name = object.name || '';
    this.dictionaryColumn = object.dictionaryColumn || 'Id';
    this.versioned = object.versioned === undefined ? true : object.versioned;
    this.audited = object.audited === undefined ? false : object.audited;

    this.includes = object.includes || [];
    this.uniqueConstraints = object.uniqueConstraints || [];

    this.properties = object.properties ?
      object.properties.map(val => new Property(val)) :
      this.defaultSetOfProperties();

    this.tryMigrateProperties();

    this.relations = object.relations ? object.relations.map(val => new Relation(val)) : [];
    this.staticRows = object.staticRows || [];
  };

  store() {
    localStorage.setItem('ClassDef', JSON.stringify(this));
  };

  static restore() {
    let value = localStorage.getItem('ClassDef');
    if (value) {
      let obj = JSON.parse(value);
      return new ClassDef(obj);
    }

    return new ClassDef( {} );
  };

  clear() {
    this.name = '';
    this.dictionaryColumn = 'Id';
    this.versioned = true;
    this.audited = false;
    this.includes = [];
    this.uniqueConstraints = [];
    this.properties = [];
    this.relations = [];
    this.staticRows = [];
  };

  good() {
    if (!this.name.length) return false;
    if (!this.properties) return false;
    if (Static.badNames.some(val => val === this.name)) return false;
    if (this.properties.some(val => !val.good()) ) return false;

    return true;
  };

  tryMigrateProperties() {
    let idx = this.properties.findIndex(val => val.name === 'Id');
    if (idx === -1)
      this.properties.splice(0, 0, Property.idProperty());

    idx = this.properties.findIndex(val => val.name === 'CreatedAt');
    if (idx === -1)
      this.properties.splice(this.properties.length, 1, Property.createdAtProperty());

    let that = this;
    if (false) setTimeout(function() {
      that.versioned = !that.versioned;
      that.toggleVersioned();
    }, 100);

    return this.properties;
  };

  defaultSetOfProperties() {
    return this.versioned ?
      [ Property.idProperty(), Property.createdAtProperty() ] :
      [ Property.idProperty(), Property.createdAtProperty(), Property.updatedAtProperty() ];
  };

  addProperty(index) {
    this.properties.splice(index + 1, 0, new Property());
  };

  copyProperty(index) {
    let newProperty = new Property(this.properties[index]);
    newProperty.name += "_Copy"
    this.properties.splice(index + 1, 0, newProperty);
  };

  removeProperty(index) {
    let propertyName = this.properties[index].name;
    this.staticRows.forEach(val => delete val[propertyName]);
    this.properties.splice(index, 1);
  };

  addUpdatedAtProperty() {
    let idx = this.properties.findIndex(val => val.name === 'UpdatedAt');
    if (idx != -1) {
      return;
    }

    idx = this.properties.findIndex(val => val.name === 'CreatedAt');
    this.properties.splice(idx !== -1 ? idx + 1 : this.properties.length, 0, Property.updatedAtProperty());
  };

  removeUpdatedAtProperty() {
    let idx = this.properties.findIndex(val => val.name === 'UpdatedAt');
    if (idx != -1) {
      this.removeProperty(idx);
    }
  };

  toggleVersioned() {
    this.versioned = !this.versioned;
    if (this.versioned) {
      this.removeUpdatedAtProperty();
      this.removeRelationWithClass(new Relation(Static.updatedByUserRelation));
    } else {
      this.addUpdatedAtProperty();
      if (this.audited) this.addRelationWithClass(new Relation(Static.updatedByUserRelation));
    }
  };

  toggleAudited() {
    this.audited = !this.audited;
    if (this.audited) {
      this.addRelationWithClass(new Relation(Static.createdByUserRelation));
      if (!this.versioned) this.addRelationWithClass(new Relation(Static.updatedByUserRelation));
    } else {
      this.removeRelationWithClass(new Relation(Static.createdByUserRelation));
      this.removeRelationWithClass(new Relation(Static.updatedByUserRelation));
    }
  };

  addRelationWithClass(relation) {
    let idx = this.relations.findIndex(val => val.property === relation.property);
    if (idx != -1) {
      return;
    }

    this.relations.splice(this.relations.length, 0, relation);
  }

  removeRelationWithClass(relation) {
    let idx = this.relations.findIndex(val => val.property === relation.property);
    if (idx != -1) {
      this.relations.splice(idx, 1);
    }
  }

  addRelation(index) {
    this.relations.splice(index, 0, new Relation());
  };

  removeRelation(index) {
    this.relations.splice(index, 1);
  };

  hasOneRelations() {
    return this.relations.filter(val => val.isHasOneRelation());
  };

  hasOneRelationProperties() {
    return this.hasOneRelations()
      .map(val => ( new Property( { name: val.property, cxxType: val.getClass(), defaultValue: 'nullptr', isRelation: true } ) ));
  };

  hasOneRelationNames() {
    return this.hasOneRelations().map(val => val.property);
  };

  hasManyRelations() {
    return this.relations.filter(val => val.isHasManyRelation());
  };

  hasManyRelationProperties() {
    return this.hasManyRelations()
      // .map(val => ( new Property( { name: val.property, cxxType: 'TObjArray', defaultValue: 'NULL', isRelation: true } ) ));
      .map(val => ( new Property( { name: val.property, cxxType: val.getClass(), defaultValue: 'nullptr', isRelation: true } ) ));
  };

  hasManyRelationNames() {
    return this.hasManyRelations().map(val => val.property);
  };

  manyToManyRelations() {
    return this.relations.filter(val => val.isManyToManyRelation());
  };

  manyToManyRelationProperties() {
    return this.manyToManyRelations()
      // .map(val => ( new Property( { name: val.property, cxxType: 'TObjArray', defaultValue: 'NULL', isRelation: true } ) ));
      .map(val => ( new Property( { name: val.property, cxxType: val.getClass(), defaultValue: 'nullptr', isRelation: true } ) ));
  };

  manyToManyRelationNames() {
    return this.manyToManyRelations().map(val => val.property);
  };


  allRelationalProperties() {
    return []
    .concat( this.hasOneRelationProperties())
    .concat( this.hasManyRelationProperties());
  };

  allRelationNames() {
    return []
    .concat( this.hasOneRelationProperties())
    .concat( this.hasManyRelationProperties())
    .map(val => val.name);
  };

  fieldDeclarationProperties() {
    return [ ]
      .concat( this.hasOneRelations()
        .map(val => ( new Property( { name: val.property + 'Id', cxxType: 'int', defaultValue: '0', searcheable: true, isRelation: true } ) )))
      .concat( this.properties );
  };

  propertyNames() {
    return this.properties.map(val => val.name);
  };

  allProperties() {
    return [ Property.idProperty() ]
      .concat( this.hasOneRelations()
        .map(val => ( new Property( { name: val.property + 'Id', cxxType: 'int', defaultValue: '0', searcheable: true, isRelation: true } ) )))
      .concat( this.properties.filter(val => !val.isIdProperty()) );
  };

  allAtomicProperties() {
    return [ Property.idProperty() ]
      .concat( this.hasOneRelations()
        .map(val => ( new Property( { name: val.property + 'Id', cxxType: 'int', defaultValue: '0', searcheable: true, isRelation: true } ) )))
      .concat( this.properties.filter(val => !val.isPointer() && !val.isStlType() && !val.isRootClass() && !val.isIdProperty()) );
  };

  allArrayProperties() {
    return this.allProperties().filter(val => val.isArray());
  };

  allNonArrayProperties() {
    return this.allProperties().filter(val => !val.isArray() && !val.isStlType());
  };

  constructorProperties() {
    return [ Property.idProperty() ]
    .concat( this.hasOneRelations()
      .map(val => ( new Property( { name: val.property + 'Id', cxxType: 'int', defaultValue: '0', searcheable: true, isRelation: true } ) )))
    .concat( this.hasOneRelationProperties())
    .concat( this.hasManyRelationProperties())
    .concat( this.properties.filter(val => !val.isIdProperty()) );
  };

  arrayDimProperties() {
    return this.properties.filter(val => val.cxxType == 'Int_t').map(val => val.name);
  };

  allSearcheableProperties() {
    return this.hasOneRelations()
      .map(val => ( new Property( { name: val.property + 'Id', cxxType: 'int', searcheable: true, isRelation: true, defaultValue: -1 } ) ))
    .concat( this.properties.filter(val => val.searcheable && !val.isIdProperty()) );
  };

  viProperties() {
    return this.allProperties().map(function (val) {
        let defaultValue = val.defaultValue;
        let jsonType = val.jsonType;
        //  || val.cxxType == "std::string" || val.cxxType == "string" || val.cxxType == "TString"
        if (val.isRootClass()) {
          defaultValue = "\"\"";
        }

        if (val.isStlVector() || val.isStlVector2D()) {
          jsonType = "array";
          if (val.isStlVector2D()) {
            jsonType += "2";
          }
          jsonType += "." + val.jsonType.toLowerCase();
        }

        return { Property: val.name, Type: jsonType, Searcheable: val.searcheable, DefaultValue: defaultValue, Root: val.isRootClass() };
      });
      // .concat(this.allArrayProperties().map(val => ( {Property: val.name, Type: "array.double", Searcheable: false, DefaultValue: val.defaultValue, Root: false} )));
  };

  viRelations() {
    return this.relations.map(val => ( {Class: val.getClass(), Type: val.type, Property: val.property} ));
  };

  propertyPointers() {
    return this.properties.filter(val => val.isPointer());
  };

  stlVectorProperties() {
    return this.properties.filter(val => val.isStlVector());
  };

  stlVector2DProperties() {
    return this.properties.filter(val => val.isStlVector2D());
  };

  stlProperties() {
    return this.properties.filter(val => val.isStlType());
  };

  rootClassProperties() {
    return this.properties.filter(val => val.isRootClass());
  };

  allPointers() {
    return this.relations.map(val => ( new Property( { name: val.property, cxxType: val.getClass() + '*', defaultValue: 'NULL' } ) ))
      .concat( this.propertyPointers() );
  };

  allNonArrayPointers() {
    return this.relations.map(val => ( new Property( { name: val.property, cxxType: val.getClass() + '*', defaultValue: 'NULL' } ) ))
      .concat( this.propertyPointers().filter(val => !val.isArray()) );
  };

  relationClasses() {
    return this.relations.map(val => val.getClass());
  };

  allFriendlyTypes() {
    return Static.types.map((type) => type.friendlyType)
      .concat(this.includes.map(val => val));
  };

  allTypes() {
    return Static.types.map((type) => type.cxxType)
      .concat(this.includes.map(val => val));
  };

  headerFileName() {
    return this.name + '.h';
  };

  cxxFileName() {
    return this.name + '.cxx';
  };

  cxxBaseTemplateClass() {
    return this.versioned ? 'BaseClassVersioned' : 'BaseClass';
  };

  cxxBaseTemplateClassNamespaced() {
    return this.versioned ? 'TGenBase::BaseClassVersioned' : 'TGenBase::BaseClass';
  };

  resourceHeaderFileName() {
    return this.name + 'Resource.h';
  };

  resourceCxxFileName() {
    return this.name + 'Resource.cxx';
  };

  controlFileName() {
    return this.name + '.ctl';
  };

  classFileName() {
    return this.name + '.zip';
  };

  classLibFileName() {
    return 'ClassLibrary.zip';
  };

  resourceName() {
    return this.name + 'Resource';
  };

  resourceDefineMacro() {
    return this.resourceName().toUpperCase() + "_H";
  };

  classInstanceVariableName() {
    return this.name.charAt(0).toLowerCase() + this.name.slice(1) + 'Instance';
  };

  classVariableNamePlural() {
    return this.name.charAt(0).toLowerCase() + this.name.slice(1) + 's';
  };

  classVariableNameSingular() {
    return this.name.charAt(0).toLowerCase() + this.name.slice(1);
  };

  lowerCaseName() {
    return this.name.toLowerCase();
  };

  explodedName() {
    return this.name.explode();
  };

  columnsTemplateAll() {
    let props = [ Property.idProperty() ]
        .map(val => { return { title: val.name, field: val.name,  visible:true, type: val.friendlyType, relation: this.name, relationType: 'Has One' }; } )
      .concat( this.hasOneRelations()
        .map(val => { return { title: val.property + 'Id', field: val.property + 'Id', visible:true, type: 'Integer', relation: val.getClass(), relationType: val.type }; } ) )

      .concat( [...this.hasManyRelations(), ...this.manyToManyRelations() ]
        .map(val => { return { title: val.property, field: val.property, visible:true, type: 'Integer', relation: val.getClass(), relationType: val.type }; } ) )

      .concat( this.properties.filter(val => !val.isPointer() && !val.isRootClass() && !val.isIdProperty() && !val.isUserIdProperty())
        .map(val => { return { title: val.name, field: val.name,  visible:true, type: val.friendlyType, subType: val.subType, date: val.isTimeStamp() }; } )
      // .concat( this.properties.filter(val => val.isUserIdProperty())
      //   .map(val => { return { title: val.name, field: val.name,  visible:true, type: 'Integer', subType: '', relation: 'User' }; } ) )
      );

    props.forEach(val => {
      if (!val.date) delete val.date;

      if (Property.specialPropertyNames().some(name => name === val.field)) {
        val.visible = false;
      } else {
        val.visible = true;
      }
    });

    return props;
  };

  columnsTemplateSingle() {
    let props = [ Property.idProperty() ]
        .map(val => { return { title: val.name, field: val.name,  visible:true, type: val.friendlyType, subType: val.subType, relation: this.name, relationType: 'Has One', default: val.jsDefaultValue }; } )
      .concat( this.hasOneRelations()
        .map(val => { return { title: val.property + 'Id', field: val.property + 'Id',  visible:true, type: 'Integer', subType: '', relation: val.getClass(), relationType: val.type, default: 0 }; } ) )

      .concat( [...this.hasManyRelations(), ...this.manyToManyRelations() ]
        .map(val => { return { title: val.property, field: val.property, visible:true, type: 'Integer', subType: '', relation: val.getClass(), relationType: val.type }; } ) )

      .concat( this.properties.filter(val => !val.isPointer() && !val.isRootClass() && !val.isIdProperty() && !val.isUserIdProperty())
        .map(val => { return { title: val.name, field: val.name,  visible:true, type: val.friendlyType, subType: val.subType, readonly: val.isSpecial(), date: val.isTimeStamp(), regex: val.regex, default: val.jsDefaultValue }; } ) )
      .concat( this.properties.filter(val => val.isRootClass())
        .map(val => { return { title: val.name, field: val.name,  visible:true, type: val.friendlyType, subType: val.subType, readonly: true, root: true, opt: '', default: '""' }; } )
      // .concat( this.properties.filter(val => val.isUserIdProperty())
      //   .map(val => { return { title: val.name, field: val.name,  visible:true, type: 'Integer', subType: '', relation: 'User', default: val.jsDefaultValue }; } ) )
      );

    props.forEach(val => {
      if (!val.date) delete val.date;

      if (Property.specialPropertyNames().some(name => name === val.field)) {
        val.visible = false;
      } else {
        val.visible = true;
      }
    });

    return props;
  };

  dictionaryColumns() {
    return this.allProperties().
      filter(val => val.isIdProperty() || val.isString()).
      map(val => val.name);
  }

  // Validation
  regexProperties() {
    return this.properties.filter(val => val.isRegexProperty());
  };

  // python
  pythonClassFileName() {
    return this.name + '.py';
  };

  pythonClassPath() {
    return this.name + '.py';
  };

  pythonProperties() {
    return []
      .concat( this.hasOneRelations()
        .map(val => ( new Property( { name: val.property + 'Id', cxxType: 'int', defaultValue: '0', searcheable: true, isRelation: true, pythonDefaultValue: '0' } ) )))
      .concat( this.properties );
  };

  // laravel
  laravelModelProperties() {
    // return this.properties;
    return [ Property.idProperty() ]
        .map(val => { return { name: val.name, phpDefaultValue: val.phpDefaultValue, phpDbType: val.phpDbType }; } )
      .concat( this.hasOneRelations()
        .map(val => { return { name: val.property + 'Id', phpDefaultValue: 0, phpDbType: 'integer' }; } ) )
      .concat( this.properties.filter(val => !val.isPointer() && !val.isRootClass() && !val.isIdProperty())
        .map(val => { return { name: val.name, phpDefaultValue: val.phpDefaultValue, phpDbType: val.phpDbType, isArray: val.isArray() }; } ) )
      .concat( this.properties.filter(val => val.isRootClass())
        .map(val => { return { name: val.name, phpDefaultValue: '\'\'', phpDbType: 'text' }; } ) );
  };

  laravelModelArrayProperties() {
    return this.properties.filter(val => val.isArray())
      .map(val => { return { name: val.name, phpDefaultValue: val.phpDefaultValue, phpDbType: val.phpDbType, isArray: true }; } );
    // return []
    //   .concat( this.properties.filter(val => val.isArray())
    //     .map(val => { return { name: val.name, phpDefaultValue: val.phpDefaultValue, phpDbType: val.phpDbType, isArray: true }; } ) );
  };

  laravelMigrationName(name) {
    // let date = new Date();
    // let datePrefix = `${date.getFullYear()}_${("0" + (date.getMonth()+1)).slice(-2)}_${("0" + date.getDate()).slice(-2)}`;
    // let datePrefix = `2019_03_15`;
    return `2019_03_15_000000_create_${this.name.snakeCase()}_table.php`;
  };

  laravelPivotMigrationName() {
    return `2019_03_15_000000_create_many_to_many_pivots_table.php`;
  };

  availableStaticRows() {
    return this.allAtomicProperties()
      .filter(val => val.name !== 'CreatedAt' && val.name !== 'UpdatedAt')
      .map(val => val.name);
  };

  addStaticRow(index) {
    let object = {};
    this.availableStaticRows().forEach(val => object[val] = '');
    this.staticRows.splice(index+1, 0, object);
  };

  copyStaticRow(index) {
    this.staticRows.splice(index+1, 0, JSON.parse(JSON.stringify(this.staticRows[index])));
  };

  removeStaticRow(index) {
    this.staticRows.splice(index, 1);
  };

  staticRowInputToPhp(propertyName, propertyValue) {
    let property = this.allProperties().find(val => val.name === propertyName);
    if (property.friendlyType === 'String' || property.isStlType() || property.isRootClass()) {
      if (!(/^\".*\"$/.exec(propertyValue))) {
        return '"'+propertyValue+'"';
      }
    }

    return propertyValue;
  };
}

export default ClassDef
