import Static from '@/js/Static.js'

class Property {
  constructor(object) {
    if (!object)
      object = {};
    this.name = object.name || '';
    this.friendlyType = object.friendlyType || '';
    this.cxxType = this.tryMigrateCxxType(object.cxxType) || '';
    this.subType = object.subType || '';
    this.defaultValue = this.tryMigrateDefaultValue(object.defaultValue) || '';
    this.searcheable = object.searcheable || false;
    this.isRelation = object.isRelation || false;
    this.importedClass = object.importedClass || false;
    this.regex = object.regex || '';

    this.pythonDefaultValue = object.pythonDefaultValue || this.getPythonDefaultValue(this.cxxType) || '';
    this.jsDefaultValue = object.jsDefaultValue || this.getJsDefaultValue(this.cxxType) || '';
    this.phpDefaultValue = object.phpDefaultValue || this.getPhpDefaultValue(this.friendlyType) || '';
    this.phpDbType = object.phpDbType || this.getPhpDbType(this.friendlyType) || '';
  };

  good() {
    if (!this.name.length) return false;
    if (!this.friendlyType.length) return false;
    if (!this.cxxType.length) return false;

    return true;
  };

  tryMigrateCxxType(type) {
    if (!type)
      return undefined;

    switch (type) {
      case "string": case "TString": return "std::string";
      case "Int_t": return "int";
      case "UInt_t": return "uint";
      case "Float_t": return "float";
      case "Double_t": return "double";
      case "Bool_t": return "bool";
      case "ValTimeStamp": return "TimeStamp";
      case "std::vector<Int_t>": return "std::vector<int>";
      case "std::vector<Double_t>": return "std::vector<double>";
      case "std::vector< std::vector<Int_t> >": return "std::vector< std::vector<int> >";
      case "std::vector< std::vector<Double_t> >": return "std::vector< std::vector<double> >"

      default: return type;
    }
  }

  tryMigrateDefaultValue(val) {
    if (!val)
      return undefined;

    switch (val) {
      case "ValTimeStamp::GetBOT()": return "TimeStamp(0)";
      case "ValTimeStamp()": return "TimeStamp()";
      case "kFALSE": return "false";
      case "kTRUE": return "true";
      case "NULL": return "nullptr";

      default: return val;
    }
  }

  getPythonDefaultValue() {
    if (this.importedClass)
      return '""';

    if (this.isStlType())
      return '[]';

    let def = Static.types.filter(type => type.cxxType == this.cxxType);
    if (def && def.length) {
       return def[0].pythonDefaultValue;
    }

    return null;
  };

  getJsDefaultValue() {
    if (this.importedClass)
      return '""';

    if (this.isStlType())
      return '[]';

    let def = Static.types.filter(type => type.cxxType == this.cxxType);
    if (def && def.length) {
       return def[0].jsDefaultValue;
    }

    return null;
  };

  getPhpDefaultValue() {
    if (this.importedClass)
      return '\'\'';

    if (this.isStlType())
      return '[]';

    let def = Static.types.filter(type => type.cxxType == this.cxxType);
    if (def && def.length) {
       return def[0].phpDefaultValue;
    }

    return null;
  };

  getPhpDbType() {
    if (this.importedClass)
      return 'text';

    if (this.isStlType())
      return 'text';

    let def = Static.types.filter(type => type.cxxType == this.cxxType);
    if (def && def.length) {
       return def[0].phpDbType;
    }

    return null;
  };

  isPointer() {
    return this.cxxType.endsWith('*');
  };

  isString() {
    return this.cxxType === "std::string" || this.cxxType === "string";
  };

  isStringType() {
    return this.cxxType === "std::string" || this.cxxType === "TString"
        || this.cxxType === "char *" || this.cxxType === "const char *";
  };

  isArray() {
    // return this.cxxType === 'Int_t*' || this.cxxType === 'Double_t*';
    return this.isStlVector() || this.isStlVector2D();
  };

  isStlType()
  {
    return this.isStlVector() || this.isStlVector2D();
  };

  isStlVector() {
    return (this.cxxType.match(/std::vector</g) || []).length == 1;
  };

  isStlVector2D() {
    return (this.cxxType.match(/std::vector</g) || []).length == 2;
  };

  isTimeStamp() {
    return this.cxxType == 'TimeStamp' || this.cxxType == 'ValTimeStamp';
  };

  isRootClass() {
    return this.importedClass;
  };

  isRegexProperty() {
    return this.isString() && this.regex.length;
  }

  jsonArrayName() {
    return this.name + 'Arr';
  };

  tempArrayName() {
    return 'Temp' + this.name;
  };

  pureType() {
    return this.cxxType.replace('*', '');
  };

  pointerType() {
    return this.cxxType + "*";
  };

  uniquePtrType() {
    return "std::unique_ptr<" + this.cxxType + ">";
  };

  vectorType() {
    return "std::vector<" + this.cxxType + ">";
  };

  uniquePtrToVectorType() {
    return "std::unique_ptr< std::vector<" + this.cxxType + "> >";
  };

  getterMethod() {
    if (this.isRootClass()) {
      return `${this.uniquePtrType()} Get${this.name}() const { return copy_unique(${this.fieldName()}); }`;
    } else {
      return `${this.cxxType} Get${this.name}() const { return ${this.fieldName()}; }`;
    }
  };

  setterMethod(thin = false) {
    if (this.isRootClass()) {
      return `void Set${this.name}(${this.cxxType} value) { ${this.fieldName()} = ${this.uniquePtrType()}(new ${this.cxxType}(value)); }`
     + `\n    void Set${this.name}(${this.uniquePtrType()} value) { ${this.fieldName()} = std::move(value); }`;
    } else {
      return `void Set${this.name}(${this.cxxType} value) { ${this.fieldName()} = value; ${this.isIdProperty() && !thin ? "SetCompId(fId); " : ""}}`;
    }
  };

  searcheableGetterMethod(className, versioned) {
    return versioned ?
      `static std::vector<${className}> GetBy${this.name}(${this.cxxType} ${this.name}, TGenBase::Pagination* pagination = nullptr, TimeStamp rid=TimeStamp());` :
      `static std::vector<${className}> GetBy${this.name}(${this.cxxType} ${this.name}, TGenBase::Pagination* pagination = nullptr);`;
  };

  copyConstructorEntry() {
    if (this.isRootClass() || (this.isRelation && (this.cxxType != 'int' && this.cxxType != 'long'))) {
      return `${this.fieldName()} = copy_unique(from.${ this.fieldName()});`;
    } else {
      return `${this.fieldName()} = from.${ this.fieldName()};`;
    }
  };

  assignmentConstructorEntry() {
    if (this.isRootClass()) {
      return `Set${this.name}(*from.Get${this.name}());`;
    } else if (this.isRelation) {
      return `${this.fieldName()} = copy_unique(from.${this.fieldName()});`
    } else {
      return `Set${this.name}(from.Get${this.name}());`;
    }
  };

  fieldDeclaration() {
    if (this.isRootClass()) {
      return `${this.uniquePtrType()} ${this.fieldName()};`;
    } else {
      return `${this.cxxType} ${this.fieldName()};`;
    }
  };

  primingStructFieldEntry() {
    if (this.isRootClass()) {
      return `${this.pointerType()} ${this.fieldName()};`;
    } else {
      return `${this.cxxType} ${this.fieldName()};`;
    }
  };

  primingAssignmentEntry(singular, plural) {
    if (this.isRootClass()) {
      return `${plural}[i].${this.fieldName()} ? ${singular}.Set${this.name}(*${plural}[i].${this.fieldName()}) : ${singular}.Set${this.name}(${this.uniquePtrType()}(nullptr));`;
    } else {
      return `${singular}.Set${this.name}(${plural}[i].${this.fieldName()});`;
    }
  };

  fieldName() {
    return 'f' + this.name;
  };

  streamerName() {
    if (this.isPointer() || this.isStlType() || this.isRootClass()) {
      return this.name + 'Streamer';
    }

    return this.fieldName();
  };

  columnName() {
    // to table case; RowId -> ROW_ID
    return this.name.replace(/([a-z][A-Z])/g, function (val) { return val[0] + '_' + val[1] }).toUpperCase();
  };

  isSpecial() {
    return this.isRelation || ['Id', 'CreatedAt', 'UpdatedAt',
      Static.createdByUserIdField.name, Static.updatedByUserIdField.name].filter(val => val === this.name).length > 0;
  };

  static specialPropertyNames() {
    return [
      Static.idField.name, Static.createdAtField.name, Static.updatedAtField.name, Static.createdByUserIdField.name, Static.updatedByUserIdField.name
    ];
  }

  static idProperty() {
    return new Property( Static.idField );
  };

  static createdAtProperty() {
    return new Property( Static.createdAtField );
  };

  static updatedAtProperty() {
    return new Property( Static.updatedAtField );
  };

  static createdByUserIdProperty() {
    return new Property( Static.createdByUserIdField );
  };

  static updatedByUserIdProperty() {
    return new Property( Static.updatedByUserIdField );
  };

  isIdProperty() {
    return this.name === 'Id';
  };

  isUserIdProperty() {
    return this.isCreatedByUserIdProperty() || this.isUpdatedByUserIdProperty();
  }

  isCreatedByUserIdProperty() {
    return this.name === Static.createdByUserIdField.name;
  }

  isUpdatedByUserIdProperty() {
    return this.name === Static.updatedByUserIdField.name;
  }

  canBeSearcheable() {
    return !this.isArray() && !this.isPointer() && !this.isRootClass() && !this.isStlType();
  };

  lowerCaseName() {
    return this.name.toLowerCase();
  };

  explodedName() {
    return this.name.explode();
  };
};

export default Property
