import os
import json
import requests
import time
import shutil
from pprint import pprint
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


def Now():
    return int(time.time())


"""
Pagination for methods returning many entries
Used both for setting up and receiving the information about paging
"""


class Pagination:
    def __init__(self, PerPage = 15, Page = 1):
        self.PerPage = PerPage
        self.CurrentPage = Page
        self.From = 0
        self.To = 0
        self.LastPage = 0
        self.Total = 0

    def ToJson(self):
        return {"per_page": self.PerPage, "page": self.CurrentPage}

    @classmethod
    def FromJson(cls, Json):
        if not Json:
            return None
        self = cls()
        self.PerPage = Json['per_page'] if 'per_page' in Json else 0
        self.CurrentPage = Json['current_page'] if 'current_page' in Json else 0
        self.From = Json['from'] if 'from' in Json else 0
        self.To = Json['to'] if 'to' in Json else 0
        self.LastPage = Json['last_page'] if 'last_page' in Json else 0
        self.Total = Json['total'] if 'total' in Json else 0
        return self

    def Print(self):
        pprint(vars(self))


"""
Query building interface
Allows to chain multiple calls to handle complex search criteria
"""


class QueryBuilder:
    def __init__(self):
        self.Query = []

    def Reset(self):
        self.Query = []


    def Where(self, column, op, value):
        self.Query.append({'Name': 'Where', 'Column': column, 'Operator': op, 'Value': value})
        return self

    def OrWhere(self, column, op, value):
        self.Query.append({'Name': 'OrWhere', 'Column': column, 'Operator': op, 'Value': value})
        return self

    def WhereBetween(self, column, values):
        self.Query.append({'Name': 'WhereBetween', 'Column': column, 'Value': values})
        return self

    def OrWhereBetween(self, column, values):
        self.Query.append({'Name': 'OrWhereBetween', 'Column': column, 'Value': values})
        return self

    def WhereNotBetween(self, column, values):
        self.Query.append({'Name': 'WhereNotBetween', 'Column': column, 'Value': values})
        return self

    def OrWhereNotBetween(self, column, values):
        self.Query.append({'Name': 'OrWhereNotBetween', 'Column': column, 'Value': values})
        return self

    def WhereIn(self, column, values):
        self.Query.append({'Name': 'WhereIn', 'Column': column, 'Value': values})
        return self

    def WhereNotIn(self, column, values):
        self.Query.append({'Name': 'WhereNotIn', 'Column': column, 'Value': values})
        return self

    def OrWhereIn(self, column, values):
        self.Query.append({'Name': 'OrWhereIn', 'Column': column, 'Value': values})
        return self

    def OrWhereNotIn(self, column, values):
        self.Query.append({'Name': 'OrWhereNotIn', 'Column': column, 'Value': values})
        return self

    def OrderBy(self, column, direction):
        self.Query.append({'Name': 'OrderBy', 'Column': column, 'Value': direction})
        return self

    def Latest(self):
        self.Query.append({'Name': 'Latest'})
        return self

    def Oldest(self):
        self.Query.append({'Name': 'Oldest'})
        return self

    def First(self):
        self.Query.append({'Name': 'First'})
        return self

    def Last(self):
        self.Query.append({'Name': 'Last'})
        return self

    def Skip(self, amount):
        self.Query.append({'Name': 'Skip', 'Value': amount})
        return self

    def Take(self, amount):
        self.Query.append({'Name': 'Take', 'Value': amount})
        return self

    def Limit(self, amount):
        self.Query.append({'Name': 'Limit', 'Value': amount})
        return self

    def Execute(self, cls, pagination=None, Rid=Now()):
        if type(cls.__bases__[0]) == type(BaseClass):
            return cls.GetWithQuery(self, pagination)
        elif type(cls.__bases__[0]) == type(BaseClassVersioned):
            return cls.GetWithQuery(self, pagination, Rid)

        raise Exception("Unsupported class")


"""
Static thin HTTP client interface to TGenBase
Initialize it with a config file, where the TGenBase service endpoint URI is specified
The access token will persist on disc on successful authentication
"""


class Client:
    ConfigPath = "TGenBaseClientConfig.json"
    AccessToken = ""
    ServerURI = ""
    SchemaVersion = "1.0.0"
    ClientVersion = "1.0.0"

    def __init__(self):
        pass

    @staticmethod
    def Init():
        if os.path.exists(Client.ConfigPath):
            with open(Client.ConfigPath, 'r') as config_file:
                config = json.load(config_file)
                Client.ServerURI = config.get('ServerURI', '')
                Client.AccessToken = config.get('AccessToken', '')
                Client.SchemaVersion = config.get('SchemaVersion', '1.0.0')
                Client.ClientVersion = config.get('ClientVersion', '1.0.0')

    @staticmethod
    def StoreSession():
        with open(Client.ConfigPath, 'w+') as config_file:
            config = { 'ServerURI': Client.ServerURI,
                       'AccessToken': Client.AccessToken,
                       'SchemaVersion': Client.SchemaVersion,
                       'ClientVersion': Client.ClientVersion }
            json.dump(config, config_file, indent=2)

    @staticmethod
    def PostJsonRequest(ClassName, Method, RequestJson, pagination=None):
        if (Client.ServerURI == "") or (Client.ServerURI is None):
            Client.Init()
        if (Client.ServerURI == "") or (Client.ServerURI is None):
            raise Exception("Server URI was not set")
        if (Client.AccessToken == "") or (Client.AccessToken is None):
            if not (ClassName == "User" and Method == "Login"):
                raise Exception("Access token was not provided, you have to log in first")

        reqJson = RequestJson.copy()
        if pagination:
            reqJson.update(pagination.ToJson())

        headers = { "auth-token": Client.AccessToken,
                    "Schema-Version": Client.SchemaVersion,
                    "Client-Version": Client.ClientVersion,
                    "Client-Type": "Python",
                    "Content-Type": "application/json",
                    "X-Requested-With": "XMLHttpRequest" }
        response = requests.post(Client.ServerURI + '/' + ClassName + '/' + Method,
                                 json=reqJson,
                                 headers=headers,
                                 verify=False)

        if response.headers.get("Schema-Version") != Client.SchemaVersion:
            print('TGenBase.Client.PostJsonRequest: Schema version mismatch: Server has {} Client has {}'
                  .format(response.headers.get("Schema-Version"), Client.SchemaVersion))

        if response.status_code == 404:
            return None

        if response.status_code == 401:
            Client.AccessToken = ""

        if response.status_code >= 500:
            raise Exception(response.reason)

        responseJson = response.json()
        if responseJson['error'] is not None:
            raise Exception(responseJson['error'])

        if pagination:
            pagination.__dict__ = Pagination.FromJson(responseJson).__dict__

        return responseJson['data']

    @staticmethod
    def DownloadFile(Url, FileName=""):
        if FileName == "":
            FileName = Url.split('/')[-1]

        with requests.get(Url, stream=True, verify=False) as response:
            if response.status_code != 200:
                raise Exception(response.reason)

            with open(FileName, 'wb') as file:
                shutil.copyfileobj(response.raw, file)

        return FileName

    @staticmethod
    def UploadFile(FileName):
        if (Client.ServerURI == "") or (Client.ServerURI is None):
            Client.Init()
        if (Client.ServerURI == "") or (Client.ServerURI is None):
            raise Exception("Server URI was not set")
        if (Client.AccessToken == "") or (Client.AccessToken is None):
            raise Exception("Access token was not provided, you have to log in first")

        headers = { "auth-token": Client.AccessToken,
                    "Schema-Version": Client.SchemaVersion,
                    "Client-Version": Client.ClientVersion,
                    "Client-Type": "Python",
                    "X-Requested-With": "XMLHttpRequest" }
        with requests.post(Client.ServerURI + '/File/Upload',
                           files={'file': (FileName, open(FileName, 'rb'))},
                           headers=headers,
                           verify=False) as response:

            if response.headers.get("Schema-Version") != Client.SchemaVersion:
                print('TGenBase.Client.PostJsonRequest: Schema version mismatch: Server has {} Client has {}'
                      .format(response.headers.get("Schema-Version"), Client.SchemaVersion))

            if response.status_code == 401:
                Client.AccessToken = ""

            if response.status_code >= 500:
                raise Exception(response.reason)

            responseJson = response.json()
            if responseJson['error'] is not None:
                raise Exception(responseJson['error'])

            return responseJson['data']


"""
Base TGenBase class to inherit from.
Implements routines and methods common to all other classes.
"""


class BaseClass:
    def __init__(self, Id=-1):
        self.Id = Id
        self.CreatedAt = 0
        self.UpdatedAt = 0

    def Print(self):
        pprint(vars(self))

    def ToJson(self):
        return self.__dict__

    def ToJsonString(self):
        return json.dumps(self.__dict__, indent=2)

    @classmethod
    def ToJsonArray(cls, Array):
        return json.dumps([element.ToJson() for element in Array], indent=2)

    @classmethod
    def FromJson(cls, Json):
        if not Json:
            return None
        self = cls()
        self.__dict__ = Json
        return self

    @classmethod
    def FromJsonString(cls, String):
        Json = json.loads(String)
        if type(Json) is list:
            return cls.FromJsonArray(Json)

        if type(Json) is dict:
            return cls.FromJson(Json)

        raise Exception('Wrong json')

    @classmethod
    def FromJsonArray(cls, Array):
        return [cls.FromJson(Json) for Json in Array or []]

    @classmethod
    def GetById(cls, Id):
        return cls.FromJson(Client.PostJsonRequest(cls.__name__, "GetById", {'Id': Id}))

    @classmethod
    def GetAll(cls, pagination=None):
        return cls.FromJsonArray(Client.PostJsonRequest(cls.__name__, "GetAll", {}, pagination))

    def GetAllVersions(self):
        return self.FromJsonArray(Client.PostJsonRequest(self.__class__.__name__, "GetAllVersions", {'Id': self.Id}))

    @classmethod
    def GetArray(cls, Ids, pagination=None):
        return cls.FromJsonArray(Client.PostJsonRequest(cls.__name__, "GetArray", {'Ids': Ids}, pagination))

    @classmethod
    def GetWithQuery(cls, Query, pagination=None):
        return cls.FromJsonArray(Client.PostJsonRequest(cls.__name__, "GetWithQuery", {'Query': Query.Query}, pagination))

    @classmethod
    def GetDictionary(cls, Column = ''):
        ret = Client.PostJsonRequest(cls.__name__, "GetDictionary", {'Column': Column})
        return {int(k): v for k, v in ret.items()}

    def Store(self):
        response = Client.PostJsonRequest(self.__class__.__name__, "Store", {'Object': self.ToJson()})
        self.Id = response['Id']

    @classmethod
    def StoreArray(cls, Array):
        return Client.PostJsonRequest(cls.__name__, "StoreArray", {'Array': Array})

    def Delete(self):
        self.DeleteById(self.Id)

    @classmethod
    def DeleteById(cls, Id):
        cls.FromJson(Client.PostJsonRequest(cls.__name__, "DeleteById", {'Id': Id}))

    @classmethod
    def DeleteArray(cls, Array):
        return Client.PostJsonRequest(cls.__name__, "DeleteArray", {'Ids': Array})


"""
Base TGenBase class with versioned logic to inherit from.
Implements routines and methods common to all other classes.
"""


class BaseClassVersioned:
    def __init__(self, Id=-1):
        self.Id = Id
        self.CreatedAt = 0

    def Print(self):
        pprint(vars(self))

    def ToJson(self):
        return self.__dict__

    def ToJsonString(self):
        return json.dumps(self.__dict__, indent=2)

    @classmethod
    def ToJsonArray(cls, Array):
        return json.dumps([element.ToJson() for element in Array], indent=2)

    @classmethod
    def FromJson(cls, Json):
        if not Json:
            return None
        self = cls()
        self.__dict__ = Json
        return self

    @classmethod
    def FromJsonString(cls, String):
        Json = json.loads(String)
        if type(Json) is list:
            return cls.FromJsonArray(Json)

        if type(Json) is dict:
            return cls.FromJson(Json)

        raise Exception('Wrong json')

    @classmethod
    def FromJsonArray(cls, Array):
        return [cls.FromJson(Json) for Json in Array or []]

    @classmethod
    def GetById(cls, Id, Rid=Now()):
        return cls.FromJson(Client.PostJsonRequest(cls.__name__, "GetById", {'Id': Id, 'Rid': Rid}))

    @classmethod
    def GetAll(cls, pagination=None, Rid=Now()):
        return cls.FromJsonArray(Client.PostJsonRequest(cls.__name__, "GetAll", {'Rid': Rid}, pagination))

    def GetAllVersions(self, Rid=Now()):
        return self.FromJsonArray(Client.PostJsonRequest(self.__class__.__name__, "GetAllVersions", {'Id': self.Id, 'Rid': Rid}))

    @classmethod
    def GetArray(cls, Ids, pagination=None, Rid=Now()):
        return cls.FromJsonArray(Client.PostJsonRequest(cls.__name__, "GetArray", {'Ids': Ids, 'Rid': Rid}, pagination))

    @classmethod
    def GetWithQuery(cls, Query, pagination=None, Rid=Now()):
        return cls.FromJsonArray(Client.PostJsonRequest(cls.__name__, "GetWithQuery", {'Query': Query.Query, 'Rid': Rid}, pagination))

    @classmethod
    def GetDictionary(cls, Column = '', Rid=Now()):
        ret = Client.PostJsonRequest(cls.__name__, "GetDictionary", {'Column': Column, 'Rid': Rid})
        return {int(k): v for k, v in ret.items()}

    def Store(self, Rid=Now()):
        response = Client.PostJsonRequest(self.__class__.__name__, "Store", {'Object': self.ToJson(), 'Rid': Rid})
        self.Id = response['Id']

    @classmethod
    def StoreArray(cls, Array, Rid=Now()):
        return Client.PostJsonRequest(cls.__name__, "StoreArray", {'Array': Array, 'Rid': Rid})


"""
User class inherits from TGenBase.BaseClass and implements the authentication procedure
"""


class User(BaseClassVersioned):
    def __init__(self, Id=-1):
        BaseClassVersioned.__init__(self, Id)
        self.FullName = ""
        self.Address = ""
        self.Email = ""
        self.Role = 0
        self.Status = 0
        self.CreatedAt = 0
        self.UpdatedAt = 0

    @staticmethod
    def Login(Email, Password):
        response = Client.PostJsonRequest("User", "Login", {'email': Email, 'password': Password})

        Client.AccessToken = response['auth-token']
        Client.StoreSession()
        return User.FromJson(response['auth-user'])

    def Logout(self):
        Client.PostJsonRequest("User", "Logout", {})
        Client.AccessToken = ""
        Client.StoreSession()
