#!/bin/bash

pushd "$(dirname ${BASH_SOURCE[0]})"

# set default version
export FSOFTVER=may18p1
export FROOTVER=v18.0.6
export ROOTVER=6

export WTVER=3.3.12
export NODEVER=v8.4.0
export FAIRDBVER=v1.0.6

BASEDIR=~/dbsoft
SRCDIR=$BASEDIR/src
mkdir -p $SRCDIR

pushd $BASEDIR
  BASEDIR=`pwd`
  SRCDIR=$BASEDIR/src
popd

if [[ "$OSTYPE" == "darwin"* ]]; then
  export OPENSSL_ROOT_DIR=/usr/local/opt/openssl
  WT_SSL_PREFIX="-DSSL_PREFIX=$OPENSSL_ROOT_DIR"

  NUMOFCPU=`sysctl -n hw.ncpu`
else
  NUMOFCPU=`cat /proc/cpuinfo | grep processor | wc -l`
fi

COMPILER=gcc
SED="sed -i"
MD5="md5sum"
if [[ "$OSTYPE" == "darwin"* ]]; then
  COMPILER=Clang
  SED="sed -i .back"
  MD5="md5"
fi


# Install FairSoft
if [[ -z "$SIMPATH" ]]; then
  export SIMPATH=$BASEDIR/FairSoft
  if [[ "$PATH" != *"$SIMPATH"* ]]; then
    export PATH=$SIMPATH/bin:$SIMPATH:$PATH
  fi

  pushd $SRCDIR
    git clone https://github.com/FairRootGroup/FairSoft.git FairSoft
    cd FairSoft
    git checkout -b $FSOFTVER $FSOFTVER

    cp automatic.conf automatic_db.conf
    if [ $ROOTVER -eq 6 ]; then
      $SED "s|build_root6=no|build_root6=yes|" automatic_db.conf
    fi
    $SED "s|SIMPATH_INSTALL=.*|SIMPATH_INSTALL=$SIMPATH|" automatic_db.conf

    $SED "s|compiler=|compiler=$COMPILER|" automatic_db.conf
    ./configure.sh automatic_db.conf
  popd
fi

# Install FairRoot
if [[ -z "$FAIRROOTPATH" ]]; then
  export FAIRROOTPATH=$BASEDIR/FairRoot
  if [[ "$PATH" != *"$FAIRROOTPATH"* ]]; then
    export PATH=$FAIRROOTPATH/bin:$FAIRROOTPATH:$PATH
  fi

  pushd $SRCDIR
    git clone https://github.com/FairRootGroup/FairRoot.git FairRoot
    cd FairRoot
    git checkout -b $FROOTVER $FROOTVER

    mkdir -p build && cd build
    cmake \
      -DCMAKE_CXX_COMPILER=$($SIMPATH/bin/fairsoft-config --cxx) \
      -DCMAKE_C_COMPILER=$($SIMPATH/bin/fairsoft-config --cc) \
      -DCMAKE_INSTALL_PREFIX=$FAIRROOTPATH \
      ..

    nice make install -j$NUMOFCPU
    $SED "s|export FAIRROOTPATH=.*|export FAIRROOTPATH=\"$FAIRROOTPATH\"|" $FAIRROOTPATH/bin/FairRootConfig.sh
    $SED "s|export PATH=\"|export PATH=\"$FAIRROOTPATH/bin:$FAIRROOTPATH:|" $FAIRROOTPATH/bin/FairRootConfig.sh
    . $FAIRROOTPATH/bin/FairRootConfig.sh
  popd
fi

# Install FairDb and its dependencies
if [[ -z "$FAIRDBPATH" ]]; then
  export FAIRDBPATH=$BASEDIR/FairDb

  if [[ "$PATH" != *"$FAIRDBPATH"* ]]; then
    export PATH=$FAIRDBPATH/bin:$FAIRDBPATH:$PATH
  fi

  # Install Wt
  pushd $SRCDIR
    git clone --branch $WTVER --depth 1 https://github.com/emweb/wt.git
    cd wt
    mkdir -p build && cd build
    cmake \
      -DCMAKE_INSTALL_PREFIX=$FAIRDBPATH \
      -DBOOST_PREFIX=$SIMPATH \
      -DBUILD_EXAMPLES=OFF \
      -DBUILD_TESTS=OFF \
      $WT_SSL_PREFIX \
      ..

    nice make install -j$NUMOFCPU
  popd

  # Install node
  pushd $SRCDIR
    git clone --branch $NODEVER --depth 1 https://github.com/nodejs/node.git node
    cd node
    chmod +x configure
    ./configure
    nice make install -j$NUMOFCPU PREFIX=$FAIRDBPATH
  popd

  # Install FairDb
  pushd $SRCDIR
    git clone https://cbmgsi.githost.io/FairDb/FairDb.git FairDb
    cd FairDb
    git fetch origin
    git checkout $FAIRDBVER

    mkdir -p build && cd build
    cmake \
      -DCMAKE_CXX_COMPILER=$($SIMPATH/bin/fairsoft-config --cxx) \
      -DCMAKE_C_COMPILER=$($SIMPATH/bin/fairsoft-config --cc) \
      -DCMAKE_INSTALL_PREFIX=$FAIRDBPATH \
      -DBUILD_EXAMPLES=OFF \
      ..

    nice make install -j$NUMOFCPU

    . $FAIRDBPATH/bin/FairDbConfig.sh
  popd

  cat ~/.bashrc | grep $FAIRDBPATH/bin/FairDbConfig.sh
  if [[ $? -eq 1 ]]; then
    echo "
if [[ -f $FAIRDBPATH/bin/FairDbConfig.sh ]]; then
  . $FAIRDBPATH/bin/FairDbConfig.sh
fi
" >> ~/.bashrc
  fi
fi

# Generate docroot out of vue.js project
if [[ -d gui/vue ]]; then

  pushd gui/vue
    CKSUM=`ls -lAR src | $MD5`
    if [[ ! -f .md5 ]] || [[ "$CKSUM" != `cat .md5` ]]; then
      npm install
      npm run build
    fi
    echo "$CKSUM" > .md5
  popd

fi

# Build CMake project
mkdir -p build && pushd build
  echo SIMPATH=$SIMPATH
  echo FAIRROOTPATH=$FAIRROOTPATH
  echo FAIRDBPATH=$FAIRDBPATH
  cmake \
      -DCMAKE_CXX_COMPILER=$($SIMPATH/bin/fairsoft-config --cxx) \
      -DCMAKE_C_COMPILER=$($SIMPATH/bin/fairsoft-config --cc) \
      -DCMAKE_INSTALL_PREFIX=$FAIRDBPATH \
      ..

  nice make install -j$NUMOFCPU
popd

if [[ ! -d .git ]]; then
  git init > /dev/null
  git config user.name fairdb
  git config user.email fairdb@gsi.de
  git add . && git commit -m "Init" > /dev/null
fi

popd
