<?php

namespace App;

use Illuminate\Support\Facades\DB;
use Illuminate\Database\Eloquent\Model;

/**
 * Base model for all TGenBase classes
 * Allows updating model in-place
 */
class BaseModel extends Model
{
  /**
   * The primary key for the model.
   *
   * @var string
   */
  protected $primaryKey = 'Id';

  const CREATED_AT = 'CreatedAt';
  const UPDATED_AT = 'UpdatedAt';

  /**
   * The storage format of the model's date columns.
   *
   * @var string
   */
  protected $dateFormat = 'U';

  /**
   * Model attributes with defailt values
   *
   * @var array
   */
  protected $attributes = [
    'Id' => -1,
    'CreatedAt' => 0,
    'UpdatedAt' => 0,
  ];

  /**
   * The attributes that are mass assignable.
   *
   * @var array
   */
  protected $fillable = ['Id', 'CreatedAt', 'UpdatedAt'];

  /**
   * Indicates if created_at and updated_at timestamps are used
   *
   * @var bool
   */
  public $timestamps = true;

  /**
   * Indicates if the IDs are auto-incrementing.
   *
   * @var bool
   */
  public $incrementing = true;

  /**
   * Table name for this model
   *
   * @var strung
   */
  protected $table = '%BaseModel';

  /**
   * Default column for dictionary building
   *
   * @var string
   */
  public const dictionaryColumn = '%&';

  /**
   * Indicates if audit enabled for this class.
   *
   * @var bool
   */
  public const audit = false;

  /**
   * The attributes that should be hidden for arrays.
   *
   * @var array
   */
  protected $hidden = ['pivot'];

  /**
   * Database Query Builder interface
   * @param  int|null  $rid  not used
   * @return \Illuminate\Database\Query\Builder
   */
  public static function GetBy($rid=null) {
    $with = request('With', null);
    if ($with) {
      if (is_string($with))
        $with = [$with];

      $filtered = [];
      foreach ($with as $key => $value) {
        if (method_exists(static::class, explode(":", $value)[0])) {
          $filtered[] = $value;
        }
      }

      return static::query()->with($filtered);
    }
    else
      return static::query();
  }

  /**
   * Get entity by Id
   *
   * @param  int  $Id
   * @return \App\BaseModel|mixed
   */
  public static function GetById($Id) {
    return Cache::remember(static::class, 'GetById', $Id, 600, function () use ($Id) {
      return static::GetBy()->where('Id', $Id)->get()->first();
    });
  }

  /**
   * Get all entities
   *
   * param  int|null $perPage
   * @return array
   */
  public static function GetAll() {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetAll', [$perPage,$page], 600, function () use ($perPage) {
      if (is_null($perPage))
        return static::GetBy()->get();
      else
        return static::GetBy()->paginate($perPage);
    });
  }

  /**
   * Get the only version (latest) of model
   *
   * @param  int  $Id
   * @return array
   */
  public static function GetAllVersions($Id) {
    $result = static::GetById($Id);
    if ($result)
      return [$result];

    return [];
  }

  /**
   * Get array of entities with array of Ids
   *
   * param  array $Ids
   * @return array
   */
  public static function GetArray($Ids) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetArray', [$Ids,$perPage,$page], 600, function () use ($Ids, $perPage) {
      if (is_null($perPage))
        return static::GetBy()->whereIn('Id', $Ids)->get();
      else
        return static::GetBy()->whereIn('Id', $Ids)->paginate($perPage);
    });
  }

  /**
   * Get array of entities with array of Ids
   *
   * param  int|null $Query
   * @return array
   */
  public static function GetWithQuery($Query) {
    $Query = collect($Query);
    \App\QueryBuilder::check($Query);

    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetWithQuery', [$Query,$perPage,$page], 600, function () use ($Query, $perPage) {
      $ret = static::GetBy();
      \App\QueryBuilder::process($Query, $ret);
      if (is_null($perPage))
        return $ret->get();
      else
        return $ret->paginate($perPage);
    });
  }

  /**
   * Get dictionary of 'Id' => $Column as 'Id' => 'Name' for this entity
   *
   * @param  string|null $Column
   * @return array
   */
  public static function GetDictionary($Column = null) {
    if (is_null($Column) || $Column === '')
      $Column = static::dictionaryColumn;

    return Cache::remember(static::class, 'GetDictionary', $Column, 600, function () use ($Column) {
      $inst = new static;
      if (in_array($Column, $inst->getHidden()))
        $Column = '%&'.$Column;

      $dictionary = [];
      static::GetBy()->get(['Id', $Column])->each(function($val) use ($Column, &$dictionary) {
        $dictionary[$val['Id']] = $val[$Column];
      });
      return $dictionary;
    });
  }

  /**
   * Store an instance of entity
   * If entity Id is not set or equals -1 a new Id will be allocated
   *
   * @return \App\BaseModel|mixed
   */
  public function Store() {
    Cache::flush(static::class);

    unset($this->CreatedAt, $this->UpdatedAt);

    $this->exists = true;
    if (!isset($this->Id) || is_null($this->Id) || $this->Id == -1) {
      unset($this->Id);
      $this->exists = false;

      if (!app()->runningInConsole() && static::audit) {
        $this->CreatedByUserId = auth()->user()->Id;
        $this->UpdatedByUserId = auth()->user()->Id;
      }

      $this->save();
      return $this;
    }

    if (!app()->runningInConsole() && static::audit) {
      unset($this->CreatedByUserId);
      $this->UpdatedByUserId = auth()->user()->Id;
    }

    return $this->updateOrCreate(['Id' => $this->Id], $this->attributesToArray());
  }

  /**
   * Store an eloquent collection of entity objects
   * If entity Id is not set or equals -1 a new Id will be allocated
   *
   * @param  \Illuminate\Support\Collection  $Collection
   * @return null  nothing
   */
  public static function StoreCollection($Collection) {
    if (!$Collection->count()) {
      return collect([]);
    }

    Cache::flush(static::class);

    $inserts = $Collection->filter(function ($val, $key) {
      return !isset($val->Id) || is_null($val->Id) || $val->Id == -1;
    })->map(function ($val) {
      unset($val->Id, $val->CreatedAt, $val->UpdatedAt);

      $val->exists = false;

      if (!app()->runningInConsole() && static::audit) {
        $val->CreatedByUserId = auth()->user()->Id;
        $val->UpdatedByUserId = auth()->user()->Id;
      }

      return $val->getAttributes();
    });

    DB::transaction(function () use (&$inserts, &$Collection) {
      static::insert($inserts->toArray());

      $Collection->each(function (&$val) {
        // updates
        if (isset($val->Id) && !is_null($val->Id) && $val->Id != -1) {
          unset($val->CreatedAt, $val->UpdatedAt);

          if (!app()->runningInConsole() && static::audit) {
            unset($val->CreatedByUserId);
            $this->UpdatedByUserId = auth()->user()->Id;
          }

          $val->updateOrCreate(['Id' => $val->Id], $val->attributesToArray());
        }
      });

      return $Collection;
    }, 3);

    return null;
  }

  /**
   * Store an array of entity associated arrays
   * If entity Id is not set or equals -1 a new Id will be allocated
   *
   * @param  array  $Array
   * @return array
   */
  public static function StoreArray($Array) {
    return static::StoreCollection(collect($Array));
  }

  /**
   * Delete the model instance
   * Already implemented in the Eloquent model
   */
  // public function Delete() {
  //   static::delete();
  // }

  /**
   * Delete the model instance with Id
   *
   * @param  int  $Id
   */
  public static function DeleteById($Id) {
    static::destroy([$Id]);
  }

  /**
   * Delete an array of models with array of Ids
   *
   * @param  array  $Id
   */
  public static function DeleteArray($Ids) {
    static::destroy($Ids);
  }
}
