<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;

use App\Http\Controllers\Controller;
use App\Http\Controllers\UserController;
use App\User;

class AdminController extends Controller
{
  /**
   * Local user registration
   * LDAP user import happens on login, see UserController::Login
   *
   * param  Request $request
   * @return array
   */
  public function RegisterUser(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'user' => 'required',
      'user.Email' => 'required|min:3',
      'user.FullName' => 'required|min:3',
      'user.Address' => 'required|min:3',
      'password' => 'required|min:8',
    ]);

    $user = new User((array)request('user'));

    // Check email is taken
    if (User::GetByEmail($user->Email)->first()) {
      return \App\Http\Response::Error('User with this email was already registered.', 422);
    }

    $user->Id = -1;
    $user->Password = Hash::make(request('password'));

    try {
      return \App\Http\Response::Data($user->Store());
    } catch (\Throwable $e) {
      return \App\Http\Response::Error($e->getMessage(), 422);
    }
  }

  public function ChangeUserPassword(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Id' => 'required|numeric',
      'password' => 'required|min:8',
    ]);

    $user = User::GetById(request('Id'));
    if (!$user) {
      return \App\Http\Response::Error('User not found', 422);
    }

    $current_user = auth()->user();
    if ($current_user->Role < 1000 && $current_user->Id != $user->Id) {
      throw new \App\Exceptions\AppException(422, 'Not allowed');
    }

    $user->Password = Hash::make(request('password'));
    $user->Store();
    return \App\Http\Response::Data(null);
  }

  public function ChangeUser(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Id' => 'required|numeric',
      'user' => 'required',
    ]);

    $user = User::GetById(request('Id'));
    if (!$user) {
      return \App\Http\Response::Error('User not found', 422);
    }

    $current_user = auth()->user();
    if ($current_user->Role < 1000 && $current_user->Id != $user->Id) {
      throw new \App\Exceptions\AppException(403, 'Not allowed');
    }

    $modified = new User((array)request('user'));

    // Check email is taken
    $otherUser = User::GetByEmail($modified->Email)->first();
    if ($otherUser && $otherUser->Id !== $user->Id) {
      return \App\Http\Response::Error('This email is already registered to another user.', 422);
    }

    $user->FullName = $modified->FullName;
    $user->Address = $modified->Address;
    $user->Email = $modified->Email;
    if ($current_user->Role === 1000) {
      $user->Status = $modified->Status;
      $user->Role = $modified->Role;
    }

    try {
      return \App\Http\Response::Data($user->Store());
    } catch (\Throwable $e) {
      return \App\Http\Response::Error($e->getMessage(), 422);
    }
  }

  public function GetUser(Request $request) {
    return (new UserController)->GetUserById($request);
  }

  public function GetConfig(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Keys' => 'nullable|array',
    ]);
    $keys = request('Keys', null);
    if (is_null($keys)) {
      $keys = \App\Helpers\Environment::allowedKeys;
    }

    return \App\Http\Response::Data(\App\Helpers\Environment::getArray($keys), 200, JSON_FORCE_OBJECT);
  }

  public function SetConfig(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Config' => 'required|array',
    ]);

    return \App\Http\Response::Data(\App\Helpers\Environment::setArray(request('Config')));
  }
}
