<?php

namespace App;

/**
 * Interface for building queries on the client side
 */
class QueryBuilder
{

  /**
   * Supported queries for GetWithQuery method
   *
   * @var \Illuminate\Support\Collection
   */
  const supportedQueries = [
    'Where', 'OrWhere', 'WhereBetween', 'OrWhereBetween',
    'WhereNotBetween', 'OrWhereNotBetween', 'WhereIn', 'WhereNotIn',
    'OrWhereIn', 'OrWhereNotIn', 'OrderBy', 'Latest',
    'Oldest', 'First', 'Last', 'Skip', 'Take', 'Limit',
    'With', 'WhereHas'
  ];

  public static function check(&$Query) {
    $supported = collect(static::supportedQueries);
    $Query->pluck('Name')->each(function ($item, $key) use ($supported) {
      if (!$supported->contains($item)) {
          throw new \App\Exceptions\AppException(422, "Unsupported query: ".$item);
      }
    });
  }

  public static function process(&$Query, &$ret, $modelTable = null) {
    $Query->each(function ($item, $key) use (&$ret, $modelTable) {
      if (!is_null($modelTable) && isset($item["Column"]) && $item["Column"] === "Id") {
        $item["Column"] = $modelTable.".Id";
      }

      switch($item["Name"]) {
        case "Where":
          if (isset($item["Query"]) && count($item["Query"])) {
            $ret = $ret->where(function($nestedQuery) use ($item) {
              $subQuery = collect($item["Query"]);
              static::process($subQuery, $nestedQuery);
            });
            break;
          }

          $ret = $ret->where($item["Column"], $item["Operator"], $item["Value"]); break;
        case "OrWhere":
          if (isset($item["Query"]) && count($item["Query"])) {
            $ret = $ret->orWhere(function($nestedQuery) use ($item) {
              $subQuery = collect($item["Query"]);
              static::process($subQuery, $nestedQuery);
            });
            break;
          }

          $ret = $ret->orWhere($item["Column"], $item["Operator"], $item["Value"]); break;
        case "WhereBetween":
          $ret = $ret->whereBetween($item["Column"], $item["Value"]); break;
        case "OrWhereBetween":
          $ret = $ret->orWhereBetween($item["Column"], $item["Value"]); break;
        case "WhereNotBetween":
          $ret = $ret->whereNotBetween($item["Column"], $item["Value"]); break;
        case "OrWhereNotBetween":
          $ret = $ret->orWhereNotBetween($item["Column"], $item["Value"]); break;
        case "WhereIn":
          $ret = $ret->whereIn($item["Column"], $item["Value"]); break;
        case "WhereNotIn":
          $ret = $ret->WwereNotIn($item["Column"], $item["Value"]); break;
        case "OrWhereIn":
          $ret = $ret->orWhereIn($item["Column"], $item["Value"]); break;
        case "OrWhereNotIn":
          $ret = $ret->orWhereNotIn($item["Column"], $item["Value"]); break;
        case "OrderBy":
          $order = $item["Value"];
          if (is_null($order) || $order === "" || ($order !== "asc" && $order !== "desc"))
            $order = "asc";
          $ret = $ret->orderBy($item["Column"], $order); break;

        case "Latest":
          $ret = $ret->latest(); break;
        case "Oldest":
          $ret = $ret->oldest(); break;
        case "First":
          $ret = $ret->take(1); break;
        case "Last":
          $ret = $ret->latest()->take(1); break;

        case "Skip":
          $ret = $ret->skip($item["Value"]); break;
        case "Take":
          $ret = $ret->take($item["Value"]); break;
        case "Limit":
          $ret = $ret->limit($item["Value"]); break;

        case "With":
          $ret = $ret->with($item["Value"]); break;

        case "WhereHas":
          if (!(isset($item["Query"]) || !count($item["Query"]))) {
            throw new \App\Exceptions\AppException(422, 'Nested query is not set or empty');
          }

          $ret = $ret->whereHas($item["Relation"], function(&$nestedQuery) use ($item) {
            $modelTable = $nestedQuery->getModel()->getTable();
            $subQuery = collect($item["Query"]);
            static::process($subQuery, $nestedQuery, $modelTable);
          }); break;
      }
    });
  }
}
