<?php

namespace App\Routes;

use Illuminate\Support\Facades\Route;

/**
 * Helper class to provide standard routes for versioned entitiy models
 * as well as set of special routes for UserController and AdminController
 */
class BaseRoutes
{
    /**
     * Define standard routes for model with
     * optional closure to define additional routes
     *
     * @param string  $model
     * @param Closure|null  $moreRoutes
     * @return \Illuminate\Routing\Router|\Illuminate\Routing\RouteRegistrar
     */
    public static function for($model, \Closure $moreRoutes = null){
      return Route::group(
        [
            'prefix' => $model,
        ], function() use ($model, $moreRoutes) {
          Route::post('/GetById', $model.'Controller@GetById')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetAll', $model.'Controller@GetAll')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetAllVersions', $model.'Controller@GetAllVersions')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetArray', $model.'Controller@GetArray')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetWithQuery', $model.'Controller@GetWithQuery')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetDictionary', $model.'Controller@GetDictionary')->middleware(['auth:token','status','role:guest']);
          Route::post('/Store', $model.'Controller@Store')->middleware(['auth:token','status','role:operator']);
          Route::post('/StoreArray', $model.'Controller@StoreArray')->middleware(['auth:token','status','role:operator']);
          Route::post('/DeleteById', $model.'Controller@DeleteById')->middleware(['auth:token','status','role:operator']);
          Route::post('/DeleteArray', $model.'Controller@DeleteArray')->middleware(['auth:token','status','role:operator']);

          if ($moreRoutes) $moreRoutes();
      });
    }

    /**
     * Define standard routes for versioned entity model with
     * optional closure to define additional routes
     *
     * @param string  $model
     * @param Closure|null  $moreRoutes
     * @return \Illuminate\Routing\Router|\Illuminate\Routing\RouteRegistrar
     */
    public static function forVersioned($model, \Closure $moreRoutes = null){
      return Route::group(
        [
            'prefix' => $model,
        ], function() use ($model, $moreRoutes) {
          Route::post('/GetById', $model.'Controller@GetById')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetAll', $model.'Controller@GetAll')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetAllVersions', $model.'Controller@GetAllVersions')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetArray', $model.'Controller@GetArray')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetWithQuery', $model.'Controller@GetWithQuery')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetDictionary', $model.'Controller@GetDictionary')->middleware(['auth:token','status','role:guest']);
          Route::post('/Store', $model.'Controller@Store')->middleware(['auth:token','status','role:operator']);
          Route::post('/StoreArray', $model.'Controller@StoreArray')->middleware(['auth:token','status','role:operator']);

          if ($moreRoutes) $moreRoutes();
      });
    }

    /**
     * Define standard routes for special versioned entity model User
     *
     * @return \Illuminate\Routing\Router|\Illuminate\Routing\RouteRegistrar
     */
    public static function forUser(){
      return Route::group(
        [
            'prefix' => 'User',
        ], function() {
          Route::post('/GetById', 'UserController@GetById')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetAll', 'UserController@GetAll')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetAllVersions', 'UserController@GetAllVersions')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetArray', 'UserController@GetArray')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetWithQuery', 'UserController@GetWithQuery')->middleware(['auth:token','status','role:guest']);
          Route::post('/GetDictionary', 'UserController@GetDictionary')->middleware(['auth:token','status','role:guest']);
          Route::post('/Store', 'UserController@Store')->middleware(['auth:token','status','role:admin']);
          Route::post('/StoreArray', 'UserController@StoreArray')->middleware(['auth:token','status','role:admin']);

          Route::post('/GetByFullName', 'UserController@GetByFullName')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetByAddress', 'UserController@GetByAddress')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetByEmail', 'UserController@GetByEmail')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetByRole', 'UserController@GetByRole')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetByStatus', 'UserController@GetByStatus')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetByCreatedAt', 'UserController@GetByCreatedAt')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetByUpdatedAt', 'UserController@GetByUpdatedAt')->middleware(['auth:token','status','role:admin']);

          Route::post('/Login', 'UserController@Login');
          Route::post('/Logout', 'UserController@Logout')->middleware(['auth:token','status','role:guest']);;
          Route::post('/Get', 'UserController@Get')->middleware(['auth:token','status','role:guest']);;
          Route::post('/ChangeUserPassword', 'UserController@ChangeUserPassword')->middleware(['auth:token','status','role:guest']);
          Route::post('/ChangeUser', 'UserController@ChangeUser')->middleware(['auth:token','status','role:guest']);
          Route::post('/RegisterUser', 'UserController@RegisterUser');
      });
    }

    /**
     * Define functional routes for admin workplace
     *
     * @return \Illuminate\Routing\Router|\Illuminate\Routing\RouteRegistrar
     */
    public static function forAdmin(){
      return Route::group(
        [
            'prefix' => 'Admin',
        ], function() {
          Route::post('/RegisterUser', 'AdminController@RegisterUser')->middleware(['auth:token','status','role:admin']);
          Route::post('/ChangeUserPassword', 'AdminController@ChangeUserPassword')->middleware(['auth:token','status','role:admin']);
          Route::post('/ChangeUser', 'AdminController@ChangeUser')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetUser', 'AdminController@GetUser')->middleware(['auth:token','status','role:admin']);
          Route::post('/GetConfig', 'AdminController@GetConfig')->middleware(['auth:token','status','role:admin']);
          Route::post('/SetConfig', 'AdminController@SetConfig')->middleware(['auth:token','status','role:admin']);
      });
    }

    /**
     * Define functional routes for admin workplace
     *
     * @return \Illuminate\Routing\Router|\Illuminate\Routing\RouteRegistrar
     */
    public static function forFiles() {
      $prefix = env('UPLOADS_PATH', 'up');
      Route::post($prefix.'/{path}', 'FileController@ServeFile')->where('path', '(.*)')->middleware(['auth:token','status','role:guest']);
      Route::get($prefix.'/{path}', 'FileController@ServeFile')->where('path', '(.*)')->middleware(['auth:token','status','role:guest']);

      return Route::group(
        [
            'prefix' => 'File',
        ], function() {
          Route::post('/GetById', 'FileController@GetById')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetAll', 'FileController@GetAll')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetAllVersions', 'FileController@GetAllVersions')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetArray', 'FileController@GetArray')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetWithQuery', 'FileController@GetWithQuery')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetDictionary', 'FileController@GetDictionary')->middleware(['auth:token','status','role:operator']);
          Route::post('/Store', 'FileController@Store')->middleware(['auth:token','status','role:operator']);
          Route::post('/StoreArray', 'FileController@StoreArray')->middleware(['auth:token','status','role:operator']);
          Route::post('/DeleteById', 'FileController@DeleteById')->middleware(['auth:token','status','role:operator']);
          Route::post('/DeleteArray', 'FileController@DeleteArray')->middleware(['auth:token','status','role:operator']);

          Route::post('/Upload', 'FileController@UploadFile')->middleware(['auth:token','status','role:operator']);

          Route::post('/GetByFileName', 'FileController@GetByFileName')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetByDescription', 'FileController@GetByDescription')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetByChecksum', 'FileController@GetByChecksum')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetByCreatedAt', 'FileController@GetByCreatedAt')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetByUpdatedAt', 'FileController@GetByUpdatedAt')->middleware(['auth:token','status','role:operator']);
          Route::post('/GetByUserId', 'FileController@GetByUserId')->middleware(['auth:token','status','role:operator']);
      });
      // Route::get('/', 'FileController@GetByUserId')->middleware(['auth:token','status','role:operator']);
    }
}
